/*
 * Beaver.h
 *
 */

#ifndef PROTOCOLS_BEAVER_H_
#define PROTOCOLS_BEAVER_H_

#include <vector>
#include <array>
using namespace std;

#include "Replicated.h"
#include "Processor/Data_Files.h"

template<class T> class SubProcessor;
template<class T> class MAC_Check_Base;
class Player;

/**
 * Beaver multiplication
 */
template<class T>
class Beaver : public ProtocolBase<T>
{
    vector<T> shares;
    vector<typename T::open_type> opened;
    vector<typename T::open_type> previously_opened;
    vector<array<T, 3>> triples;
    typename vector<typename T::open_type>::iterator it;
    typename vector<typename T::open_type>::iterator previous_it;
    typename vector<array<T, 3>>::iterator triple;
    Preprocessing<T>* prep;
    typename T::MAC_Check* MC;
    std::vector<std::vector<int>> binomial_coefficients;

    void adjust_binomial_coefficients(int n);

    static constexpr int prepared_binomial_coefficients = 20;
public:
    static const bool uses_triples = true;

    Player& P;

    Beaver(Player& P) : prep(0), MC(0), P(P) { adjust_binomial_coefficients(-1); }

    Player& branch();

    void init_mul(SubProcessor<T>* proc);
    void init_mul(Preprocessing<T>& prep, typename T::MAC_Check& MC);
    typename T::clear prepare_mul(const T& x, const T& y, int n = -1);

    void prepare_power(T variable, int exponent);
    T finalize_power(int exponent);
    void finalize_prefix_power(register_span<T> result);

    void prepare_product(register_span<T const> factors);
    void continue_product(std::size_t factor_count);
    T finalize_product(std::size_t factor_count);
    void finalize_prefix_product(register_span<T> result);

    void prepare_monomial(register_span<T const> variables);
    T finalize_monomial(std::span<int const> exponents);

    void prepare_polynomial(register_span<T const> variables);
    T finalize_polynomial(register_span<typename T::clear const> coefficients, std::span<int const> exponents);

    void exchange();
    void second_exchange();
    T finalize_mul(int n = -1);

    void start_exchange();
    void stop_exchange();

    int get_n_relevant_players() { return 1 + T::threshold(P.num_players()); }
};

#endif /* PROTOCOLS_BEAVER_H_ */
