#!/usr/bin/env python3

import fire
import os
import subprocess

def main(source=".", destination="lcov/coverage.info", reset=False, destructive=False):
    if reset:
        subprocess.check_call(["lcov", "--directory", source, "--zerocounters"])
        return

    destination = os.path.abspath(destination)
    destination_directory = os.path.dirname(destination)

    os.makedirs(destination_directory, exist_ok=True)

    subprocess.check_call(["lcov", "--capture", "--directory", source, "--output-file", destination + ".raw"])
    subprocess.check_call(["lcov", "--remove", destination + ".raw", "/usr/*", "--output-file", destination])
    subprocess.check_call(["lcov", "--list", destination])
    subprocess.check_call(["genhtml", "--output-directory", destination_directory, destination])

    if destructive:
        subprocess.check_call(["lcov", "--directory", source, "--zerocounters"])

if __name__ == "__main__":
    fire.Fire(main)